/***
 **
 *  File: Kmer.h
 *  Created: Dec 6, 2009
 *
 *  Author: Xiao Yang <isuxyang@gmail.com>
 *
 *  Copyright 2009 Xiao Yang, Karin Dorman, Srinivas Aluru
 *
 *  This file is part of Reptile.
 *
 *  Reptile is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Reptile is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with Libpnorm. If not, see <http://www.gnu.org/licenses/>.
 *
 */ 

#ifndef _KMER_H
#define	_KMER_H

#include "util.h"
#include "Seq.h"


typedef struct KInfo{
    char* repaddr;
    int goodCnt;
    int cnt;
    KInfo(){};
    KInfo(char* p, int c1, int c2): repaddr(p), goodCnt(c1), cnt(c2){};
}kc_s;

typedef struct KC{
    uint64_t ID;
    int goodCnt;
    int cnt;
    KC(){};
    KC(uint64_t myID, int c1, int c2): ID(myID), goodCnt(c1), cnt(c2){};
}kc_t;

// used by sort, compare length K cstr
struct Kstrcomp {
    int K;
    Kstrcomp (int kvalue): K(kvalue) {};

    bool operator() (const std::pair<char*, char*>& e1,
                     const std::pair<char*, char*>& e2) const {

        if (strncmp(e1.first, e2.first, K) < 0)
            return true;
        return false;

    }
};

struct Knumcomp {
    bool operator() (const kc_t& e1, const kc_t& e2) const {
        if (e1.ID < e2.ID) return true;
        return false;
    }
};

class Kmer {

protected:
    std::vector<kc_s> elem_;
    bool goodQuality(char* qAddr, const Para& myPara);
public:
    Kmer();
    Kmer(const Kmer& orig);
    virtual ~Kmer();
    void getKmer(const Seq& mySeq, int step, const Para& myPara);
    const std::vector<kc_s>& getElem () const {
        return elem_;
    }
    void clear(){
        elem_.clear();
    }
};




#endif	/* _KMER_H */

